from django.conf import settings
from django.contrib import contenttypes
from django.db import models

import mptt

from mayan.apps.acls.classes import ModelPermission

from .links import link_object_copy
from .menus import menu_object
from .permissions import permission_object_copy


class ModelCopy:
    _registry = {}
    _lazy = {}

    @staticmethod
    def method_instance_copy(self, values=None):
        model_copy = ModelCopy.get(model=self._meta.model)
        return model_copy.copy(instance=self, values=values)

    @classmethod
    def add_fields_lazy(cls, model, **kwargs):
        cls._lazy.setdefault(
            model, []
        )
        cls._lazy[model].append(kwargs)

    @classmethod
    def get(cls, model):
        return cls._registry[model]

    def __init__(
        self, model, acl_bind_link=True, condition=None, bind_link=False,
        excludes=None, extra_kwargs=None, register_permission=False
    ):
        self.condition = condition
        self.excludes = excludes or {}
        self.extra_kwargs = extra_kwargs or {}
        self.fields = []
        self.fields_copy = []
        self.fields_foreign_keys = []
        self.fields_generic_related = []
        self.fields_many_to_many = []
        self.fields_reverse_related = []
        self.fields_many_to_many_reverse_related = []
        self.fields_related_one_to_one = []
        self.fields_unique = []

        self.model = model
        self.__class__._registry[model] = self
        model.add_to_class(
            name='copy_instance', value=ModelCopy.method_instance_copy
        )
        if bind_link:
            menu_object.bind_links(
                links=(link_object_copy,), sources=(model,), position=99
            )

        if register_permission:
            ModelPermission.register(
                bind_link=acl_bind_link, model=model,
                permissions=(permission_object_copy,)
            )

        for entry in self.__class__._lazy.get(model, ()):
            self.add_fields(**entry)
            self.__class__._lazy.get(model).pop()

    def __str__(self):
        return str(self.label)

    def _evaluate_field_get_for_field(self, field, instance, value, values):
        context = {'instance': instance}
        context.update(values)

        field_value_gets = self.field_value_gets.get(field, None)
        if field_value_gets:
            related_model = self.model._meta.get_field(
                field_name=field
            ).related_model or self.model._meta.get_field(
                field_name=field
            ).model
            final_filter = {}
            for key, value in field_value_gets.items():
                final_filter[key] = value.format(**context)

            value = related_model._meta.default_manager.get(**final_filter)

        field_value_templates = self.field_value_templates.get(field, None)
        if field_value_templates:
            value = field_value_templates.format(**context) or None

        return value

    def add_fields(
        self, field_names, field_value_gets=None, field_values=None,
        field_value_templates=None, unique_conditional=None
    ):
        self.unique_conditional = unique_conditional or {}
        self.field_value_gets = field_value_gets or {}
        self.field_value_templates = field_value_templates or {}
        self.field_values = field_values or {}

        for field_name in field_names:
            self.fields.append(field_name)

            field = self.model._meta.get_field(field_name=field_name)

            if isinstance(field, models.fields.reverse_related.OneToOneRel):
                self.fields_related_one_to_one.append(field_name)
            elif isinstance(field, models.fields.reverse_related.ManyToOneRel):
                self.fields_reverse_related.append(field_name)
            elif isinstance(field, models.fields.related.ForeignKey):
                self.fields_foreign_keys.append(field_name)
            elif isinstance(field, contenttypes.fields.GenericRelation):
                self.fields_generic_related.append(field_name)
            elif isinstance(field, contenttypes.fields.GenericForeignKey):
                self.fields_foreign_keys.append(field_name)
            elif isinstance(field, models.fields.related.ManyToManyField):
                self.fields_many_to_many.append(field_name)
            elif isinstance(field, models.fields.reverse_related.ManyToManyRel):
                self.fields_many_to_many_reverse_related.append(field.related_name)
            else:
                if field.unique:
                    self.fields_unique.append(field_name)
                else:
                    self.fields_copy.append(field_name)

    def copy(self, instance, values=None):
        values = values or {}

        if self.excludes:
            if self.model._meta.default_manager.filter(pk=instance.pk, **self.excludes).exists():
                return

        if isinstance(self.model, mptt.models.MPTTModelBase):
            node_map = {instance.pk: None}
            self.field_value_templates = {
                'parent_id': '{parent_id}'
            }

            for source_node in instance.get_descendants(include_self=True):
                if source_node.parent:
                    values['parent_id'] = node_map[source_node.parent.pk]
                else:
                    values['parent_id'] = ''

                new_node = self._copy(
                    instance=source_node, values=values,
                    _get_or_create=self.extra_kwargs.get(
                        'get_or_create', False
                    )
                )
                if not values['parent_id']:
                    result = new_node
                node_map[source_node.pk] = new_node.pk

            return result
        else:
            return self._copy(instance=instance, values=values)

    def _copy(self, instance, values=None, _get_or_create=False):
        context = {'instance': instance}
        context.update(values)

        new_model_dictionary = {}

        # Static values.
        for field, value in self.field_values.items():
            new_model_dictionary[field] = value

        # Static values templates.
        for field, value in self.field_value_templates.items():
            result = value.format(**context) or None
            new_model_dictionary[field] = result

        # Base fields whose values are copied.
        for field in self.fields_copy:
            value = values.get(
                field, getattr(instance, field)
            )

            value = self._evaluate_field_get_for_field(
                field=field, instance=instance, value=value, values=values
            )
            new_model_dictionary[field] = value

        # Base fields with unique values.
        for field in self.fields_unique:
            base_value = getattr(instance, field)
            counter = 1

            while True:
                value = '{}_{}'.format(base_value, counter)
                queryset = self.model._meta.default_manager.filter(
                    **{field: value}
                )
                if not queryset.exists():
                    break

                counter += 1

            value = self._evaluate_field_get_for_field(
                field=field, instance=instance, value=value, values=values
            )
            new_model_dictionary[field] = value

        # Foreign keys.
        for field in self.fields_foreign_keys:
            value = values.get(
                field, getattr(instance, field)
            )

            value = self._evaluate_field_get_for_field(
                field=field, instance=instance, value=value, values=values
            )
            new_model_dictionary[field] = value

        # Fields that are given an unique value if a condition is met.
        for field in self.unique_conditional:
            if self.unique_conditional[field](
                instance=instance,
                new_instance_dictionary=new_model_dictionary
            ):
                base_value = getattr(instance, field)
                counter = 1

                while True:
                    value = '{}_{}'.format(base_value, counter)
                    queryset = self.model._meta.default_manager.filter(
                        **{field: value}
                    )
                    if not queryset.exists():
                        break

                    counter += 1

                value = self._evaluate_field_get_for_field(
                    field=field, instance=instance, value=value,
                    values=values
                )
                new_model_dictionary[field] = value

        if _get_or_create:
            new_instance, created = self.model._meta.default_manager.get_or_create(
                **new_model_dictionary
            )
        else:
            new_instance = self.model(**new_model_dictionary)
            new_instance.save()

        # Many to many fields added after instance creation.
        for field in self.fields_many_to_many:
            getattr(new_instance, field).set(
                getattr(instance, field).all()
            )

        # Many to many reverse related fields added after instance creation.
        for field in self.fields_many_to_many_reverse_related:
            getattr(new_instance, field).set(
                getattr(instance, field).all()
            )

        # Reverse related.
        for field in self.fields_reverse_related:
            related_field = self.model._meta.get_field(field_name=field)
            related_field_name = related_field.field.name

            for related_instance in getattr(instance, field).all():
                values.update(
                    {related_field_name: new_instance}
                )
                related_instance.copy_instance(
                    values=values
                )

        # Reverse related one to one.
        for field in self.fields_related_one_to_one:
            related_field = self.model._meta.get_field(field_name=field)
            related_field_name = related_field.field.name

            getattr(instance, field).copy_instance(
                values={related_field_name: new_instance}
            )

        # Generic relations.
        for field in self.fields_generic_related:
            related_field = self.model._meta.get_field(field_name=field)
            related_field_name = 'content_object'

            for related_instance in getattr(instance, field).all():
                related_instance.copy_instance(
                    values={related_field_name: new_instance}
                )

        return new_instance

    def get_fields_verbose_names(self):
        result = []

        for field_name in self.fields:
            field = self.model._meta.get_field(field_name=field_name)

            verbose_name = getattr(field, 'verbose_name', None)

            if not verbose_name and field.related_model:
                verbose_name = field.related_model._meta.verbose_name

            result.append(verbose_name)

        return result

    def test_condition(self, instance):
        if self.condition:
            return self.condition(instance=instance)
        else:
            return True


class MissingItem:
    _registry = []

    @classmethod
    def get_all(cls):
        return cls._registry

    @classmethod
    def get_missing(cls):
        result = []
        for item in cls.get_all():
            if item.condition():
                result.append(item)
        return result

    def __init__(self, label, condition, description, view):
        self.condition = condition
        self.description = description
        self.label = label
        self.view = view
        self.__class__._registry.append(self)


class PropertyHelper:
    """
    Makes adding fields using __class__.add_to_class easier.
    Each subclass must implement the `constructor` and the `get_result`
    method.
    """
    @staticmethod
    @property
    def constructor(source_object):
        return PropertyHelper(source_object)

    def __init__(self, instance):
        self.instance = instance

    def __getattr__(self, name):
        return self.get_result(name=name)

    def get_result(self, name):
        """
        The method that produces the actual result. Must be implemented
        by each subclass.
        """
        raise NotImplementedError


class UpstreamSetting:
    def __init__(self, name, default):
        self.default = default
        self.name = name
        self.upstream_setting_collection = None

    def do_kwargs_capture(self, **kwargs):
        value = kwargs.pop(self.name.lower(), self.default)
        setattr(settings, self.name, value)


class UpstreamSettingCollection:
    def __init__(self, name):
        self.name = name
        self.setting_list = []

    def do_setting_add(self, **kwargs):
        setting = UpstreamSetting(**kwargs)
        setting.upstream_setting_collection = self
        self.setting_list.append(setting)

    def do_kwargs_capture(self, **kwargs):
        for setting in self.setting_list:
            setting.do_kwargs_capture(**kwargs)



import elasticsearch_dsl

from django.db import models

from ...search_query_types import (
    QueryTypeExact, QueryTypeFuzzy, QueryTypeGreaterThan,
    QueryTypeGreaterThanOrEqual, QueryTypeLessThan, QueryTypeLessThanOrEqual,
    QueryTypePartial, QueryTypeRange, QueryTypeRangeExclusive,
    QueryTypeRegularExpression
)
from ...value_transformations import (
    ValueTransformationAccentReplace, ValueTransformationAtReplace,
    ValueTransformationCasefold, ValueTransformationHyphenReplace,
    ValueTransformationHyphenStrip, ValueTransformationToBoolean,
    ValueTransformationToDateTime, ValueTransformationToDateTimeISOFormat,
    ValueTransformationToDateTimeTimestamp, ValueTransformationToInteger,
    ValueTransformationToString
)

DEFAULT_ELASTICSEARCH_CLIENT_MAXSIZE = 10
DEFAULT_ELASTICSEARCH_CLIENT_SNIFF_ON_START = False
DEFAULT_ELASTICSEARCH_CLIENT_SNIFF_ON_CONNECTION_FAIL = False
DEFAULT_ELASTICSEARCH_CLIENT_SNIFFER_TIMEOUT = None
DEFAULT_ELASTICSEARCH_CLIENT_VERIFY_CERTS = True
DEFAULT_ELASTICSEARCH_HOST = 'http://127.0.0.1:9200'
DEFAULT_ELASTICSEARCH_INDICES_NAMESPACE = 'mayan'

"""
Integer field:

Django:
An integer. Values from -2147483648 to 2147483647 are safe in all
databases supported by Django.
https://docs.djangoproject.com/en/3.2/ref/models/fields/#integerfield

Django:
An integer. Values from -2147483648 to 2147483647 are safe in all
databases supported by Django.
https://docs.djangoproject.com/en/3.2/ref/models/fields/#integerfield

ElasticSearch:
A signed 32-bit integer with a minimum value of -2^31 (-2147483648)
and a maximum value of 2^31-1 (2147483647).
https://www.elastic.co/guide/en/elasticsearch/reference/current/number.html

Positive integer field:

Django:
Like an IntegerField, but must be either positive or zero (0).
Values from 0 to 2147483647 are safe in all databases supported by
Django. The value 0 is accepted for backward compatibility reasons.
https://docs.djangoproject.com/en/3.2/ref/models/fields/#positiveintegerfield
"""

DJANGO_TO_ELASTICSEARCH_FIELD_MAP = {
    models.AutoField: {
        'field': elasticsearch_dsl.field.Integer,
        'query_type_list': [
            QueryTypeExact, QueryTypePartial, QueryTypeGreaterThan,
            QueryTypeGreaterThanOrEqual, QueryTypeLessThan,
            QueryTypeLessThanOrEqual, QueryTypeRange, QueryTypeRangeExclusive
        ],
        'transformations': {
            'index': [
                ValueTransformationToInteger
            ],
            'search': [
                ValueTransformationToInteger
            ]
        }
    },
    models.BooleanField: {
        'field': elasticsearch_dsl.field.Boolean,
        'query_type_list': [
            QueryTypeExact
        ],
        'transformations': {
            'index': [
                ValueTransformationToString, ValueTransformationCasefold
            ],
            'search': [
                ValueTransformationToString, ValueTransformationToBoolean,
                ValueTransformationToString, ValueTransformationCasefold
            ]
        }
    },
    models.CharField: {
        'field': elasticsearch_dsl.field.Text,
        'query_type_list': [
            QueryTypeExact, QueryTypeFuzzy, QueryTypePartial,
            QueryTypeRegularExpression, QueryTypeGreaterThan,
            QueryTypeGreaterThanOrEqual, QueryTypeLessThan,
            QueryTypeLessThanOrEqual
        ],
        'transformations': {
            'index': [
                ValueTransformationAccentReplace,
                ValueTransformationHyphenReplace, ValueTransformationCasefold
            ],
            'search': [
                ValueTransformationAccentReplace,
                ValueTransformationHyphenReplace, ValueTransformationCasefold
            ]
        }
    },
    models.DateTimeField: {
        'field': elasticsearch_dsl.field.Date,
        'query_type_list': [
            QueryTypeExact, QueryTypeGreaterThan,
            QueryTypeGreaterThanOrEqual, QueryTypeLessThan,
            QueryTypeLessThanOrEqual, QueryTypeRange, QueryTypeRangeExclusive
        ],
        'transformations': {
            'index': [ValueTransformationToDateTimeISOFormat],
            'search': [
                ValueTransformationToDateTime,
                ValueTransformationToDateTimeTimestamp
            ]
        }
    },
    models.EmailField: {
        'field': elasticsearch_dsl.field.Keyword,
        'query_type_list': [
            QueryTypeExact, QueryTypePartial
        ],
        'transformations': {
            'index': [
                ValueTransformationHyphenReplace,
                ValueTransformationAtReplace, ValueTransformationCasefold
            ],
            'search': [
                ValueTransformationHyphenReplace,
                ValueTransformationAtReplace, ValueTransformationCasefold
            ]
        }
    },
    models.IntegerField: {
        'field': elasticsearch_dsl.field.Integer,
        'query_type_list': [
            QueryTypeExact, QueryTypeGreaterThan,
            QueryTypeGreaterThanOrEqual, QueryTypeLessThan,
            QueryTypeLessThanOrEqual, QueryTypeRange, QueryTypeRangeExclusive
        ],
        'transformations': {
            'search': [ValueTransformationToInteger]
        }
    },
    models.PositiveIntegerField: {
        'field': elasticsearch_dsl.field.Integer,
        'query_type_list': [
            QueryTypeExact, QueryTypeGreaterThan,
            QueryTypeGreaterThanOrEqual, QueryTypeLessThan,
            QueryTypeLessThanOrEqual, QueryTypeRange, QueryTypeRangeExclusive
        ],
        'transformations': {
            'search': [ValueTransformationToInteger]
        }
    },
    models.TextField: {
        'field': elasticsearch_dsl.field.Text,
        'query_type_list': [
            QueryTypeExact, QueryTypeFuzzy, QueryTypePartial,
            QueryTypeRegularExpression, QueryTypeGreaterThan,
            QueryTypeGreaterThanOrEqual, QueryTypeLessThan,
            QueryTypeLessThanOrEqual
        ],
        'transformations': {
            'index': [
                ValueTransformationAccentReplace,
                ValueTransformationHyphenReplace, ValueTransformationCasefold
            ],
            'search': [
                ValueTransformationAccentReplace,
                ValueTransformationHyphenReplace, ValueTransformationCasefold
            ]
        }
    },
    models.UUIDField: {
        'field': elasticsearch_dsl.field.Keyword,
        'query_type_list': [
            QueryTypeExact, QueryTypePartial, QueryTypeRegularExpression
        ],
        'transformations': {
            'index': [
                ValueTransformationToString, ValueTransformationHyphenStrip
            ],
            'search': [
                ValueTransformationToString, ValueTransformationHyphenStrip
            ]
        }
    }
}

MAXIMUM_API_ATTEMPT_COUNT = 10



from ..literals import TEST_WORKFLOW_INSTANCE_LOG_ENTRY_COMMENT

from .workflow_template_transition_mixins import WorkflowTemplateTransitionTestMixin


class WorkflowInstanceTestMixin(WorkflowTemplateTransitionTestMixin):
    auto_upload_test_document = False

    def _create_test_document_stub(self, *args, **kwargs):
        super()._create_test_document_stub(*args, **kwargs)
        self._inject_test_workflow_instance()

    def _create_test_workflow_instance(self):
        self._test_workflow_instance = self._test_workflow_template.launch_for(
            document=self._test_document
        )

    def _create_test_workflow_template_instance_log_entry(self):
        self._test_document.workflows.first().log_entries.create(
            comment=TEST_WORKFLOW_INSTANCE_LOG_ENTRY_COMMENT,
            transition=self._test_workflow_template_transition,
            user=self._test_case_user
        )

    def _do_test_workflow_instance_transition(self, extra_data=None):
        self._test_document.workflows.first().do_transition(
            comment=TEST_WORKFLOW_INSTANCE_LOG_ENTRY_COMMENT,
            extra_data=extra_data,
            transition=self._test_workflow_template_transition
        )

    def _inject_test_workflow_instance(self):
        self._test_workflow_instance = self._test_document.workflows.first()

    def _upload_test_document(self, *args, **kwargs):
        super()._upload_test_document(*args, **kwargs)
        self._inject_test_workflow_instance()


class DocumentWorkflowTemplateViewTestMixin(WorkflowInstanceTestMixin):
    def _request_test_document_single_workflow_template_launch_view(self):
        return self.post(
            viewname='document_states:document_single_workflow_templates_launch',
            kwargs={
                'document_id': self._test_document.pk
            }, data={
                'workflows': self._test_workflow_template.pk
            }
        )


class WorkflowInstanceAPIViewTestMixin(WorkflowInstanceTestMixin):
    def _request_test_workflow_instance_detail_api_view(self):
        return self.get(
            viewname='rest_api:workflow-instance-detail', kwargs={
                'document_id': self._test_document.pk,
                'workflow_instance_id': self._test_document.workflows.first().pk
            }
        )

    def _request_test_workflow_instance_list_api_view(self):
        return self.get(
            viewname='rest_api:workflow-instance-list', kwargs={
                'document_id': self._test_document.pk
            }
        )

    def _request_test_workflow_instance_log_entry_create_api_view(
        self, workflow_instance, extra_data=None
    ):
        data = {
            'transition_id': self._test_workflow_template_transition.pk
        }

        if extra_data:
            data.update(extra_data)

        return self.post(
            viewname='rest_api:workflow-instance-log-entry-list', kwargs={
                'document_id': self._test_document.pk,
                'workflow_instance_id': workflow_instance.pk
            }, data=data
        )

    def _request_test_workflow_instance_log_entry_list_api_view(self):
        return self.get(
            viewname='rest_api:workflow-instance-log-entry-list', kwargs={
                'document_id': self._test_document.pk,
                'workflow_instance_id': self._test_document.workflows.first().pk
            }
        )


class WorkflowInstanceLaunchAPIViewTestMixin(WorkflowInstanceTestMixin):
    def _request_test_workflow_instance_launch_api_view(self):
        return self.post(
            viewname='rest_api:workflow-instance-launch', kwargs={
                'document_id': self._test_document.pk
            }, data={
                'workflow_template_id': self._test_workflow_template.pk
            }
        )


class WorkflowInstanceLogEntryTransitionListAPIViewTestMixin(
    WorkflowInstanceTestMixin
):
    def _request_test_workflow_instance_log_entry_transition_list_api_view(self):
        return self.get(
            viewname='rest_api:workflow-instance-log-entry-transition-list',
            kwargs={
                'document_id': self._test_document.pk,
                'workflow_instance_id': self._test_workflow_instance.pk
            }
        )


class WorkflowInstanceViewTestMixin(WorkflowInstanceTestMixin):
    def _request_test_document_workflow_instance_list_view(self):
        return self.get(
            viewname='document_states:workflow_instance_list', kwargs={
                'document_id': self._test_document.pk
            }
        )

    def _request_test_workflow_instance_detail_view(self):
        return self.get(
            viewname='document_states:workflow_instance_detail',
            kwargs={
                'workflow_instance_id': self._test_workflow_instance.pk
            }
        )

    def _request_test_workflow_instance_transition_execute_view(self):
        return self.post(
            viewname='document_states:workflow_instance_transition_execute',
            kwargs={
                'workflow_instance_id': self._test_workflow_instance.pk,
                'workflow_template_transition_id': self._test_workflow_template_transition.pk
            }
        )

    def _request_test_workflow_instance_transition_selection_get_view(self):
        return self.get(
            viewname='document_states:workflow_instance_transition_selection',
            kwargs={
                'workflow_instance_id': self._test_workflow_instance.pk
            }
        )

    def _request_test_workflow_instance_transition_selection_post_view(self):
        return self.post(
            viewname='document_states:workflow_instance_transition_selection',
            kwargs={
                'workflow_instance_id': self._test_workflow_instance.pk
            }, data={
                'transition': self._test_workflow_template_transition.pk
            }
        )

    def _request_test_workflow_template_launch_view(self):
        return self.post(
            viewname='document_states:workflow_template_launch', kwargs={
                'workflow_template_id': self._test_workflow_template.pk
            }
        )


class WorkflowToolViewTestMixin(WorkflowInstanceTestMixin):
    def _request_workflow_launch_view(self):
        return self.post(
            viewname='document_states:tool_launch_workflows',
        )



import logging
import random

from flanker import mime

from django.core.files.base import ContentFile
from django.utils.encoding import force_bytes
from django.utils.translation import ugettext_lazy as _

from mayan.apps.credentials.class_mixins import BackendMixinCredentials
from mayan.apps.source_periodic.source_backends.mixins import SourceBackendMixinPeriodicCompressed

from ..source_backend_actions import SourceBackendActionEmailDocumentUpload

logger = logging.getLogger(name=__name__)


class SourceBackendMixinEmail(
    BackendMixinCredentials, SourceBackendMixinPeriodicCompressed
):
    action_class_list = (SourceBackendActionEmailDocumentUpload,)

    @classmethod
    def get_form_fields(cls):
        fields = super().get_form_fields()

        fields.update(
            {
                'host': {
                    'class': 'django.forms.CharField',
                    'label': _('Host'),
                    'kwargs': {
                        'max_length': 128
                    },
                    'required': True
                },
                'ssl': {
                    'class': 'django.forms.BooleanField',
                    'default': True,
                    'label': _('SSL'),
                    'required': False
                },
                'port': {
                    'class': 'django.forms.IntegerField',
                    'help_text': _(
                        'Typical choices are 110 for POP3, 995 for POP3 '
                        'over SSL, 143 for IMAP, 993 for IMAP over SSL.'
                    ),
                    'kwargs': {
                        'min_value': 0
                    },
                    'label': _('Port')
                },
                'store_body': {
                    'class': 'django.forms.BooleanField',
                    'default': True,
                    'help_text': _(
                        'Store the body of the email as a text document.'
                    ),
                    'label': _('Store email body'),
                    'required': False
                }
            }
        )

        return fields

    @classmethod
    def get_form_fieldsets(cls):
        fieldsets = super().get_form_fieldsets()

        fieldsets += (
            (
                _('Common email options'), {
                    'fields': (
                        'host', 'ssl', 'port', 'store_body'
                    )
                },
            ),
        )

        return fieldsets

    def process_message(self, message):
        bytes_message = force_bytes(s=message)

        message = mime.from_string(string=bytes_message)

        return self._process_message_content(message=message)

    def _process_message_content(self, message):
        counter = 1
        # Messages are tree based, do nested processing of message parts until
        # a message with no children is found, then work our way up.
        if message.parts:
            for part in message.parts:
                yield from self._process_message_content(
                    message=part
                )
        else:
            # Treat inlines as attachments, both are extracted and saved as
            # documents.

            source_metadata = {
                'email_date': message.headers.get('Date'),
                'email_delivered_to': message.headers.get('Delivered-To'),
                'email_from': message.headers.get('From'),
                'email_message_id': message.headers.get('Message-ID'),
                'email_received': message.headers.get('Received'),
                'email_subject': message.headers.get('Subject'),
                'email_to': message.headers.get('To')
            }

            if message.is_attachment() or message.is_inline():
                # Reject zero length attachments.
                if len(message.body) != 0:
                    label = message.detected_file_name or 'attachment-{}'.format(counter)
                    counter += 1

                    yield {
                        'file': ContentFile(
                            content=message.body, name=label,
                        ), 'source_metadata': source_metadata
                    }
            else:
                # If it is not an attachment then it should be a body message
                # part. Another option is to use message.is_body().
                if message.detected_content_type == 'text/html':
                    label = 'email_body.html'
                else:
                    label = 'email_body.txt'

                if self.kwargs['store_body']:
                    yield {
                        'file': ContentFile(
                            content=force_bytes(s=message.body), name=label
                        ), 'source_metadata': source_metadata
                    }

    def get_file_identifier(self):
        file_list_generator = self.get_stored_file_list()

        file_list_generator = list(file_list_generator)

        if file_list_generator:
            return random.choice(seq=file_list_generator)



from django.conf.urls import url

from .api_views import APIAnnouncementListView, APIAnnouncementView
from .views import (
    AnnouncementCreateView, AnnouncementDeleteView, AnnouncementEditView,
    AnnouncementListView
)

urlpatterns = [
    url(
        regex=r'^announcements/$', name='announcement_list',
        view=AnnouncementListView.as_view()
    ),
    url(
        regex=r'^announcements/create/$', name='announcement_create',
        view=AnnouncementCreateView.as_view()
    ),
    url(
        regex=r'^announcements/(?P<announcement_id>\d+)/delete/$',
        name='announcement_single_delete', view=AnnouncementDeleteView.as_view()
    ),
    url(
        regex=r'^announcements/multiple/delete/$',
        name='announcement_multiple_delete', view=AnnouncementDeleteView.as_view()
    ),
    url(
        regex=r'^announcements/(?P<announcement_id>\d+)/edit/$', name='announcement_edit',
        view=AnnouncementEditView.as_view()
    )
]

api_urls = [
    url(
        regex=r'^announcements/$', name='announcement-list',
        view=APIAnnouncementListView.as_view()
    ),
    url(
        regex=r'^announcements/(?P<announcement_id>[0-9]+)/$', name='announcement-detail',
        view=APIAnnouncementView.as_view()
    )
]



from django.conf import settings
from django.utils import timezone, translation
from django.utils.translation import ugettext_lazy as _
from django.views.i18n import JavaScriptCatalog

from stronghold.views import StrongholdPublicMixin

from mayan.apps.user_management.permissions import (
    permission_user_edit, permission_user_view
)
from mayan.apps.user_management.querysets import get_user_queryset
from mayan.apps.user_management.views.view_mixins import DynamicExternalUserViewMixin
from mayan.apps.views.generics import (
    SingleObjectDetailView, SingleObjectEditView
)
from mayan.apps.views.view_mixins import ExternalObjectViewMixin

from .forms import LocaleProfileForm, LocaleProfileForm_view
from .icons import (
    icon_user_locale_profile_detail, icon_user_locale_profile_edit
)


class JavaScriptCatalogPublic(StrongholdPublicMixin, JavaScriptCatalog):
    """
    Sub class of `JavaScriptCatalog` to bypass authentication and avoid
    JavaScript errors for non authentication users.
    """


class UserLocaleProfileDetailView(
    DynamicExternalUserViewMixin, ExternalObjectViewMixin,
    SingleObjectDetailView
):
    form_class = LocaleProfileForm_view
    external_object_permission = permission_user_view
    external_object_pk_url_kwarg = 'user_id'
    view_icon = icon_user_locale_profile_detail

    def get_external_object_queryset(self):
        return get_user_queryset(user=self.request.user)

    def get_extra_context(self, **kwargs):
        return {
            'form': LocaleProfileForm_view(
                instance=self.external_object.locale_profile
            ),
            'object': self.external_object,
            'read_only': True,
            'title': _('Locale profile for user: %s') % self.external_object
        }

    def get_object(self):
        return self.external_object.locale_profile


class UserLocaleProfileEditView(
    DynamicExternalUserViewMixin, ExternalObjectViewMixin,
    SingleObjectEditView
):
    form_class = LocaleProfileForm
    external_object_permission = permission_user_edit
    external_object_pk_url_kwarg = 'user_id'
    view_icon = icon_user_locale_profile_edit

    def form_valid(self, form):
        if self.is_current_user:
            language_value = form.cleaned_data['language']
            timezone_value = form.cleaned_data['timezone']

            timezone.activate(timezone=timezone_value)
            translation.activate(language=language_value)

            if hasattr(self.request, 'session'):
                self.request.session[
                    translation.LANGUAGE_SESSION_KEY
                ] = language_value
                self.request.session[
                    settings.TIMEZONE_SESSION_KEY
                ] = timezone_value
            else:
                self.request.set_cookie(
                    settings.LANGUAGE_COOKIE_NAME, language_value
                )
                self.request.set_cookie(
                    settings.TIMEZONE_COOKIE_NAME, timezone_value
                )

        return super().form_valid(form=form)

    def get_external_object_queryset(self):
        return get_user_queryset(user=self.request.user)

    def get_extra_context(self):
        return {
            'object': self.external_object,
            'title': _(
                'Edit locale profile for user: %s'
            ) % self.external_object
        }

    def get_instance_extra_data(self):
        return {'_event_actor': self.request.user}

    def get_object(self):
        return self.external_object.locale_profile
